@@ -107,11 +107,13 @@ validate_timeout <- function (timeout, job = NULL, func_ok = FALSE) {
 
 validate_positive_number <- function (
     value, job = NULL, 
-    if_null = NULL, null_ok = TRUE, func_ok = FALSE ) {
+    if_null = NULL, null_ok = TRUE, if_na = NA, func_ok = FALSE ) {
   
   varname <- substitute(value)
   value   <- run_job_function(value, job)
   
+  if (is_na(value)) value <- if_na
+  
   if (is_null(value)     && is_true(null_ok)) return (if_null)
   if (is_function(value) && is_true(func_ok)) return (value)
   
@@ -120,6 +122,7 @@ validate_positive_number <- function (
   
   tryCatch(
     expr = {
+      if (is.logical(value)) stop()
       value <- as.numeric(value)
       stopifnot(length(value) == 1)
       stopifnot(is_true(value > 0))
@@ -175,9 +178,21 @@ validate_string <- function (value, job = NULL, cnd_ok = FALSE) {
   varname <- substitute(value)
   value   <- run_job_function(value, job)
   
-  if (is_condition(value) && is_true(cnd_ok)) value <- value$message
+  if (is_condition(value) && is_true(cnd_ok)) return (value)
+  
   if (!is_scalar_character(value) || is_na(value) || !nzchar(value))
     cli_abort(must_be(ifelse(cnd_ok, 'a string or condition', 'a string')))
+  
+  return (value)
+}
+
+validate_logical <- function (value) {
+  
+  varname <- substitute(value)
+  
+  if (!(is_true(value) || is_false(value)))
+    cli_abort(must_be('TRUE or FALSE'))
+  
   return (value)
 }
 
